import axios from 'axios'
import { Message } from 'element-ui'
import store from '@/store'
import { getToken } from '@/utils/auth'
import define from '@/utils/define'
import { Base64} from 'js-base64';

// create an axios instance
const service = axios.create({
  baseURL: process.env.VUE_APP_BASE_API, // url = base url + request url
  withCredentials: false, // send cookies when cross-domain requests
  timeout: define.timeout, // request timeout
})

// request interceptor
service.interceptors.request.use(
  config => {
    config.headers['Authorization'] = `Basic ${Base64.encode(`${'saber'}:${'saber_secret'}`)}`;
    if (store.getters.token) {
      config.headers['Blade-Auth'] = 'bearer ' + getToken()
    }
    return config
  },
  error => {
    return Promise.reject(error)
  }
)

// response interceptor
service.interceptors.response.use(
  response => {
    //获取状态码
    const status = response.data.code || response.status;
    const message = response.data.msg || response.data.error_description || '未知错误';
    const res = response.data
    let config = response.config
    let url = config.url
    if (status === 401) store.dispatch('user/logout').then(() => router.push({
      path: '/login'
    }));
    if (status !== 200) {
      console.log(status)
      Message({
        message: message,
        type: 'error',
        duration: 1500,
      })
      return Promise.reject(new Error(message))
    } else {
      return response
    }
  },
  error => {
    if (process.env.NODE_ENV === 'development') {
      console.log(error) // for debug
    }
    message({
      message: '请求出错，请重试',
      type: 'error',
      duration: 1500
    })
    return Promise.reject(error)
  }
)

export default service
